#include "ball.h"

Ball::Ball() {
   
   posX = 0.0 ;
   posY = -1.0+BALL_SIZE ;
   posZ = -2.0 ;
   speed = BALL_MIN_SPEED ;
   maxSpeed = BALL_MAX_SPEED ;
   minSpeed = BALL_MIN_SPEED ;
   size = BALL_SIZE ;
   jumping = 0 ;
   falling = 0 ;
   nbLives = 3 ;
   score = 0 ;
   sfxJump = VSDL_LoadSound( "jump.wav") ;
   sfxFall = VSDL_LoadSound( "fall.wav") ;   
}

Ball::~Ball() {
   
}

int Ball::Move( int dir) {

   if ( !falling) {
      if ( dir & BALL_MOVE_LEFT ) {
	 posX = posX - 0.01 ;
	 if ( posX < -1.0 )
	   posX = -1.0 ;
      }
      if ( dir & BALL_MOVE_RIGHT ) {
	 posX = posX + 0.01 ;
	 if ( posX > 1.0 )
	   posX = 1.0 ;
      }
      if ( dir & BALL_ACCEL ) {
	 speed *= 1.01 ;
	 if ( speed > maxSpeed )
	   speed = maxSpeed ;
      }
      
      if ( dir & BALL_BRAKE ) {
	 speed /= 1.01 ;
	 if ( speed < minSpeed )
	   speed = minSpeed ;
      }
      
      if ( dir & BALL_JUMP ) {
	 if ( (!jumping) && (!falling) ) {
	    jumping = 1 ;
	    VSDL_PlaySound( sfxJump) ;
	 }
      }
   }
   return( 1) ;
}

int Ball::Update() {
   posZ = posZ - speed ;
  
   if ( jumping) {
      jumping = jumping + 2 ;
      if ( jumping > 314 ) 
	jumping = 0 ;
      posY = -1.0 + size + 0.5*sin(jumping/100.0) ;
   }
   if ( falling ) {
      if ( speed < 0.01 ) speed = 0.01 ;
      posY = posY - speed/2.0 ;
      if ( posZ-fallZ > 0.01 ) posZ -=0.005 ;  
      if ( posZ-fallZ < -0.01) posZ +=0.005 ;
      if ( posX-fallX > 0.01 ) posX -=0.005 ;
      if ( posX-fallX < -0.01) posX +=0.005 ;
      if ( posY < -2 ) { 
	 falling = 0 ;
	 posX = 0 ;
	 posY = -1.0 + size ;
	 posZ = 0 ;
	 speed = 0 ;
      }
   }
   if ( speed < minSpeed )
     speed = minSpeed ;
   if ( speed > maxSpeed )
     speed = maxSpeed ;
   return( 1) ;
}

void Ball::Fall( float x, float z) {
   
   fallX = x ;
   fallZ = z ;
   VSDL_PlaySound( sfxFall) ;
   falling = 1 ;
}

void Ball::Glue() {
   
   speed /= 1.02 ;
}

void Ball::Display() {
   float a, b ;
   int color ;
   
   glPushMatrix() ;
   glCullFace( GL_BACK) ;
   glEnable( GL_CULL_FACE) ;
   glEnable( GL_COLOR_MATERIAL) ;
   glTranslatef( posX, posY , -BALL_Z_ADVANCE ) ;
   glRotatef( fmod(posZ*100.0,360), 1.0, 0.0, 0.0) ;
   glRotatef( posX*100.0, 0.0, 0.0, -2.0) ;
   glRotatef( 90.0, 0.0, 1.0, 0.0) ;
   color = 0 ;
   for(a=0.0;a<6.28;a+=0.628) { 
      for(b=0.0;b<6.28;b+=0.628) { /* an odd nmber of time */
	 if (color==0) {
	    glColor4f( 1.0, 1.0, 1.0, 1.0) ;
	    color = 1 ;
	 } else {
	    glColor4f( 0.0, 0.0, 0.0, 1.0) ;
	    color = 0 ;
	 }
	 glBegin( GL_QUADS) ;
	 glVertex3f(0+size*cos(a)*cos(b), 
		    0+size*cos(a)*sin(b) ,
		    0-size*sin(a)) ;
	 glVertex3f(0+size*cos(a+0.628)*cos(b), 
		    0+size*cos(a+0.628)*sin(b) ,
		    0-size*sin(a+0.628)) ;
	 glVertex3f(0+size*cos(a+0.628)*cos(b+0.628), 
		    0+size*cos(a+0.628)*sin(b+0.628) ,
		    0-size*sin(a+0.628)) ;
	 glVertex3f(0+size*cos(a)*cos(b+0.628) , 
		    0+size*cos(a)*sin(b+0.628) ,
		    0-size*sin(a)) ;
	 glEnd() ;
      }
   }
   glDisable( GL_CULL_FACE) ;
   glPopMatrix() ;
}
